@@ -351,9 +351,9 @@ int main(int argc, char* argv[])
     mreset_tty();   // 一応端末リセット
 
     saphire_init(kATConsoleApp, TRUE, runtime_script, TRUE);
-    saphire_rehash();
     saphire_job_done = main_saphire_job_done;
     readline_init();
+    saphire_rehash();
 
     /// ヒストリーを読み込む ///
     char* histfile = getenv("SAPHIRE_HISTFILE");
@@ -364,7 +364,8 @@ int main(int argc, char* argv[])
     if(histsize < 0) histsize = 1000;
 
     char* buf;
-    int history_num = 0;
+    HISTORY_STATE* history_state = history_get_history_state();
+    int history_num = history_state->length;
     while(1) {
         /// 一行読み込み ///
         readline_signal();
