@@ -34,59 +34,6 @@
 
 #include "saphire_inner.h"
 
-typedef struct {
-    string_obj* mName;
-
-    struct stat mStat;                // stat()のキャッシュ
-} sFile;
-
-sFile* sFile_new(char* name, struct stat* stat_)
-{
-    sFile* self = MALLOC(sizeof(sFile));
-
-    self->mName = STRING_NEW(name);
-    self->mStat = *stat_;
-
-    return self;
-}
-
-static BOOL sort_name(void* left, void* right)
-{
-    sFile* l = (sFile*) left;
-    sFile* r = (sFile*) right;
-
-    char* lfname = string_c_str(l->mName);
-    char* rfname = string_c_str(r->mName);
-
-    if(strcmp(lfname, ".") == 0) return TRUE;
-    if(strcmp(lfname, "..") == 0) {
-        if(strcmp(rfname, ".") == 0) return FALSE;
-
-        return TRUE;          
-    }
-    if(strcmp(rfname, ".") == 0) return FALSE;
-    if(strcmp(rfname, "..") == 0) return FALSE;
-
-    if(!S_ISDIR(l->mStat.st_mode) && !S_ISDIR(r->mStat.st_mode)
-        || S_ISDIR(l->mStat.st_mode) && S_ISDIR(r->mStat.st_mode) )
-    {
-        return strcasecmp(lfname, rfname) < 0;
-    }
-
-    if(S_ISDIR(l->mStat.st_mode))
-        return TRUE;
-    else
-        return FALSE;
-}
-
-
-
-
-
-
-
-
-
 //////////////////////////////////////////////////////////////////////
 // static variable
 //////////////////////////////////////////////////////////////////////
@@ -1826,371 +1773,6 @@ char** readline_on_complete(const char* text, int start, int end)
 
     return NULL;
 }
-// pathの最後は/をつける
-static BOOL readline_select_file_read(vector_obj* v, char* path)
-{
-    DIR* dir = opendir(path);
-    if(dir == NULL) {
-        return FALSE;
-    }
-    
-    struct dirent* entry;
-    while(entry = readdir(dir)) {
-        char fpath[PATH_MAX];
-        xstrncpy(fpath, path, PATH_MAX);
-        xstrncat(fpath, entry->d_name, PATH_MAX);
-
-        struct stat stat_;
-        memset(&stat_, 0, sizeof(struct stat));
-        if(stat(fpath, &stat_) < 0) {
-            continue;
-        }
-
-        sFile* file = sFile_new(entry->d_name, &stat_);
-
-        vector_add(v, file);
-    }
-    
-    closedir(dir);
-
-    /// ソート ///
-    vector_sort(v, sort_name);
-
-    return TRUE;
-}
-
-static void str_cut2(enum eKanjiCode code, char* mbs, int termsize, char* dest_mbs, int dest_byte)
-{
-    if(code == kUtf8) {
-        int i;
-        int n;
-
-        wchar_t* wcs 
-            = (wchar_t*)MALLOC(sizeof(wchar_t)*(termsize+1)*MB_CUR_MAX);
-        wchar_t* tmp 
-            = (wchar_t*)MALLOC(sizeof(wchar_t)*(termsize+1)*MB_CUR_MAX);
-
-        if(mbstowcs(wcs, mbs, (termsize+1)*MB_CUR_MAX) == -1) {
-            mbstowcs(wcs, "?????", (termsize+1)*MB_CUR_MAX);
-        }
-
-        i = 0;
-        while(1) {
-            if(i < wcslen(wcs)) {
-                tmp[i] = wcs[i];
-                tmp[i+1] = 0;
-            }
-            else {
-                tmp[i] = ' ';
-                tmp[i+1] = 0;
-            }
-
-            n = wcswidth(tmp, wcslen(tmp));
-            if(n < 0) {
-                xstrncpy(dest_mbs, "?????", dest_byte);
-
-                return;
-            }
-            else {
-                if(n > termsize) {
-                    tmp[i] = 0;
-
-                    if(wcswidth(tmp, wcslen(tmp)) != termsize) {
-                        tmp[i] = ' ';
-                        tmp[i+1] = 0;
-                    }
-                    break;
-                }
-            }
-
-            i++;
-        }
-
-        wcstombs(dest_mbs, tmp, dest_byte);
-    }
-    else {
-        int n;
-        BOOL tmpend = FALSE;
-        BOOL kanji = FALSE;
-        for(n=0; n<termsize && n<dest_byte-1; n++) {
-            if(kanji)
-                kanji = FALSE;
-            else if(!tmpend && is_kanji(code, mbs[n])) {
-                kanji = TRUE;
-            }
-
-            if(!tmpend && mbs[n] == 0) tmpend = TRUE;
-                        
-            if(tmpend)
-                dest_mbs[n] = ' ';
-            else
-                dest_mbs[n] = mbs[n];
-        }
-        
-        if(kanji) dest_mbs[n-1] = ' ';
-        dest_mbs[n] = 0;
-    }
-}
-
-
-static int readline_select_file(int count, int key)
-{
-    msave_ttysettings();       // 端末の設定の保存
-    msave_screen();
-    minitscr();
-
-    const int maxx = mgetmaxx();
-    const int maxy = mgetmaxy();
-
-    vector_obj* v = VECTOR_NEW(10);
-    char cwd_path[PATH_MAX];
-    mygetcwd(cwd_path, PATH_MAX);
-    xstrncat(cwd_path, "/", PATH_MAX);
-
-    char current_dir[PATH_MAX];
-    xstrncpy(current_dir, cwd_path, PATH_MAX);
-    readline_select_file_read(v, current_dir);
-
-    vector_obj* mark_files = VECTOR_NEW(10);
-
-    if(vector_size(v) > 0) {
-        int cursor = 0;
-        while(1) {
-            /// 描写 ///
-            mclear();
-            mmvprintw(0,0,"%s", current_dir);
-
-            int page = cursor/((maxy-1)*4);
-            int n = page*4*(maxy-1);
-            while(n < page*4*(maxy-1)+(maxy-1)*4 && n < vector_size(v)) {
-                sFile* file = vector_item(v, n);
-
-                int attrs = 0;
-                if(n == cursor) {
-                    attrs |= kCAReverse;
-                }
-                if(S_ISDIR(file->mStat.st_mode)) {
-                    attrs |= kCACyan;
-                    attrs |= kCABold;
-                }
-                if(attrs) mattron(attrs);
-
-                char file2[PATH_MAX];
-                str_cut2(kUtf8, string_c_str(file->mName)
-                        , maxx/4-2, file2, PATH_MAX);
-
-                int y = (n-page*4*(maxy-1)) / 4 + 1;
-                int x = ((n-page*4*(maxy-1)) % 4)*maxx/4;
-
-                char full_path[PATH_MAX];
-                xstrncpy(full_path, current_dir, PATH_MAX);
-                xstrncat(full_path, string_c_str(file->mName), PATH_MAX);
-
-                BOOL marked = FALSE;
-                int i;
-                for(i=0; i<vector_size(mark_files); i++) {
-                    string_obj* mark_file = vector_item(mark_files, i);
-
-                    if(strcmp(string_c_str(mark_file), full_path) == 0) {
-                        marked = TRUE;
-                        break;
-                    }
-                }
-
-                if(marked) 
-                    mmvprintw(y, x, "*%s", file2);
-                else
-                    mmvprintw(y, x, " %s", file2);
-
-                if(attrs) {
-                    mattroff();
-                }
-
-                n++;
-            }
-            mrefresh();
-
-            /// インプット ///
-            int meta;
-            int key = mgetch(&meta);
-
-            if(key == 14 || key == KEY_DOWN) {
-                cursor+=4;
-            }
-            else if(key == 16 || key == KEY_UP) {
-                cursor-=4;
-            }
-            else if(key == 6 || key == KEY_RIGHT) {
-                cursor++;
-            }
-            else if(key == 2 || key == KEY_LEFT) {
-                cursor--;
-            }
-            else if(key == 4 || key == KEY_NPAGE) {
-                cursor += (maxy-1)*4;
-            }
-            else if(key == 21 || key == KEY_PPAGE) {
-                cursor -= (maxy-1)*4;
-            }
-            else if(key == 12) {
-                mclear();
-                mrefresh();
-            }
-            else if(key == ' ') {
-                sFile* cursor_file = vector_item(v, cursor);
-
-                string_obj* cursor_mfile = NULL;
-
-                char cursor_path[PATH_MAX];
-                xstrncpy(cursor_path, current_dir, PATH_MAX);
-                xstrncat(cursor_path, string_c_str(cursor_file->mName), PATH_MAX);
-
-                int i;
-                for(i=0; i<vector_size(mark_files); i++) {
-                    string_obj* mfile = vector_item(mark_files, i);
-
-                    if(strcmp(string_c_str(mfile), cursor_path) == 0) {
-                        cursor_mfile = mfile;
-                        break;
-                    }
-                }
-
-                if(!cursor_mfile) {
-                    vector_add(mark_files, STRING_NEW(cursor_path));
-                }
-                else {
-                    vector_erase(mark_files
-                        , vector_index(mark_files, cursor_mfile));
-                }
-
-                cursor++;
-            }
-            else if(key == '\\') {
-                xstrncpy(current_dir, "/", PATH_MAX);
-
-                vector_clear(v);
-
-                readline_select_file_read(v, current_dir);
-                cursor = 0;
-            }
-            else if(key == '\t' || key == 'w') {
-                if(vector_size(mark_files) == 0) {
-                    sFile* cursor_file = vector_item(v, cursor);
-
-                    /// 現在のカレントディレクトリのパスなら
-                    /// ディレクトリ名を取る
-                    char cursor_path[PATH_MAX];
-                    xstrncpy(cursor_path, current_dir, PATH_MAX);
-                    xstrncat(cursor_path, string_c_str(cursor_file->mName), PATH_MAX);
-
-                    /// カレントのファイル、子ディレクトリのファイルなら
-                    /// 相対ディレクトリ
-                    if(memcmp(cursor_path
-                            , cwd_path, strlen(cwd_path)) == 0 )
-                    {
-                        char cursor_path2[PATH_MAX];
-                        memcpy(cursor_path2
-                            , cursor_path+strlen(cwd_path)
-                            , strlen(cursor_path) - strlen(cwd_path)+1);
-
-                        string_obj* str = STRING_NEW("");
-                        saphire_get_quoted_fname(cursor_path2, str);
-                        rl_insert_text(string_c_str(str));
-                    }
-                    else {
-                        string_obj* str = STRING_NEW("");
-                        saphire_get_quoted_fname(cursor_path, str);
-                        rl_insert_text(string_c_str(str));
-                    }
-                }
-                else {
-                    int j;
-                    for(j=0; j<vector_size(mark_files); j++) {
-                        char full_path[PATH_MAX];
-                        xstrncpy(full_path
-                            , string_c_str(vector_item(mark_files, j))
-                            , PATH_MAX);
-
-                        if(strstr(full_path, cwd_path) == full_path) {
-                            char full_path2[PATH_MAX];
-                            memcpy(full_path2
-                                , full_path + strlen(cwd_path) 
-                                , strlen(full_path) - strlen(cwd_path));
-                            full_path2[strlen(full_path) - strlen(cwd_path)] = 0;
-
-                            string_obj* str = STRING_NEW("");
-                            saphire_get_quoted_fname(full_path2, str);
-                            rl_insert_text(string_c_str(str));
-
-                            if(j<vector_size(mark_files)-1) 
-                                rl_insert_text(" ");
-                        }
-                        else {
-                            string_obj* str = STRING_NEW("");
-                            saphire_get_quoted_fname(full_path, str);
-                            rl_insert_text(string_c_str(str));
-
-                            if(j<vector_size(mark_files)-1) 
-                                rl_insert_text(" ");
-                        }
-                    }
-                }
-                break;
-            }
-            else if(key == 'q' || key == 3 | key == 27 || key == 7) {
-                break;
-            }
-            else if(key == 10 || key == 13) {
-                sFile* cursor_file = vector_item(v, cursor);
-
-                if(strcmp(string_c_str(cursor_file->mName), ".") != 0
-                    && S_ISDIR(cursor_file->mStat.st_mode)) 
-                {
-                    char path2[PATH_MAX];
-                    if(correct_path(current_dir
-                            , string_c_str(cursor_file->mName), path2, PATH_MAX)) 
-                    {
-                        xstrncpy(current_dir, path2, PATH_MAX);
-                        if(current_dir[strlen(current_dir)-1] != '/') {
-                            xstrncat(current_dir, "/", PATH_MAX);
-                        }
-
-                        vector_clear(v);
-
-                        readline_select_file_read(v, current_dir);
-                        cursor = 0;
-                    }
-                }
-            }
-            else if(key == 8 || key == KEY_BACKSPACE) {    // CTRL-H
-                char parent_path[PATH_MAX];
-                parentname(parent_path, PATH_MAX, current_dir);
-                xstrncpy(current_dir, parent_path, PATH_MAX);
-
-                vector_clear(v);
-
-                readline_select_file_read(v, current_dir);
-                cursor = 0;
-            }
-
-            /// 修正 ///
-            if(cursor < 0) {
-                cursor = 0;
-            }
-            if(cursor >= vector_size(v)) {
-                cursor = vector_size(v)-1;
-            }
-        }
-    }
-
-//mclear();
-//mrefresh();
-    mendwin();
-    mrestore_screen();
-    mrestore_ttysettings();    // 端末の設定の復帰
-
-    return 0;
-}
 
 static int skip_quoted(const char *s, int i, char q)
 {
@@ -2235,6 +1817,128 @@ int lftp_char_is_quoted(const char *string,int eindex)
   return (0);
 }
 
+static int readline_macro(int count, int key)
+{
+    char* tmp = MALLOC(strlen(rl_line_buffer) + 1);
+    xstrncpy(tmp, rl_line_buffer, strlen(rl_line_buffer) +1);
+    tmp[rl_point] = 0;
+
+    string_obj* ret = STRING_NEW("");
+    sRFd* rfd = sRFd_new(-1);
+    sRFd_set_buffer(rfd, tmp);
+    if(saphire_shell3(ret, "macro", "SAPHIRE_MACRO", rfd) < 0) {
+        fprintf(stderr, "run time err\n");
+        fprintf(stderr, "%s", string_c_str(gErrMsg));
+        fprintf(stderr, "please press cntrol + l....\n");
+        return 0;
+    }
+
+    rl_insert_text(string_c_str(ret));
+
+    return 0;
+}
+
+BOOL commands_history(int* rcode, vector_obj* argv, vector_obj* blocks, vector_obj* parent_blocks, sWFd* nextout, sRFd* nextin, int nexterr, char* title, BOOL input,char* sname, int sline)
+{
+    HIST_ENTRY** history = history_list();
+
+    while(*history) {
+        if(!statment_tree_internal_commands_write_nextout(nextout, (*history)->line) )
+        {
+            err_msg("signal interrupt", sname, sline);
+            return FALSE;
+        }
+        if(!statment_tree_internal_commands_write_lf(nextout,  kLF))
+        {
+            err_msg("singal interrupt", sname, sline);
+            return FALSE;
+        }
+        history++;
+    }
+
+    *rcode = 0;
+
+    return TRUE;
+}
+
+BOOL commands_add_history(int* rcode, vector_obj* argv, vector_obj* blocks, vector_obj* parent_blocks, sWFd* nextout, sRFd* nextin, int nexterr, char* title, BOOL input, char* sname, int sline)
+{
+    string_obj* str;
+    if(input) {
+        str = STRING_NEW("");
+        int result = statment_tree_internal_commands_read_nextin_oneline(nextin, str, kLF);
+        if(result < 0) {
+            err_msg("interrupt", sname, sline);
+            return FALSE;
+        }
+        else if(result == 1) {
+            *rcode = 1;
+            return TRUE;
+        }
+        string_chomp(str);
+    }
+    else if(vector_size(argv) >= 2) {
+        str = vector_item(argv, 1);
+    }
+
+    /// ヒストリに追加 ///
+    int histsize = atoi(getenv("SAPHIRE_HISTSIZE"));
+    if(histsize < 0) histsize = 1000;
+
+    add_history(string_c_str(str));
+    HISTORY_STATE* history_state = history_get_history_state();
+    int history_num = history_state->length;
+    if(history_num > histsize) {
+        HIST_ENTRY* history = remove_history(0);
+        free(history);
+    }
+
+    *rcode = 0;
+
+    return TRUE;
+}
+
+BOOL commands_read_history(int* rcode, vector_obj* argv, vector_obj* blocks, vector_obj* parent_blocks, sWFd* nextout, sRFd* nextin, int nexterr, char* title, BOOL input, char* sname, int sline)
+{
+    char* histfile = getenv("SAPHIRE_HISTFILE");
+    if(histfile) {
+        read_history(histfile);
+        *rcode = 0;
+    }
+
+    return TRUE;
+}
+
+BOOL commands_write_history(int* rcode, vector_obj* argv, vector_obj* blocks, vector_obj* parent_blocks, sWFd* nextout, sRFd* nextin, int nexterr, char* title, BOOL input, char* sname, int sline)
+{
+    char* histfile = getenv("SAPHIRE_HISTFILE");
+    if(histfile) {
+        write_history(histfile);
+        *rcode = 0;
+    }
+
+    return TRUE;
+}
+
+BOOL commands_clear_history(int* rcode, vector_obj* argv, vector_obj* blocks, vector_obj* parent_blocks, sWFd* nextout, sRFd* nextin, int nexterr, char* title, BOOL input, char* sname, int sline)
+{
+    clear_history();
+    *rcode = 0;
+
+    return TRUE;
+}
+
+BOOL commands_remove_history(int* rcode, vector_obj* argv, vector_obj* blocks, vector_obj* parent_blocks, sWFd* nextout, sRFd* nextin, int nexterr, char* title, BOOL input, char* sname, int sline)
+{
+    if(vector_size(argv) == 2) {
+        HIST_ENTRY* history = remove_history(atoi(string_c_str(vector_item(argv, 1))));
+        free(history);
+        *rcode = 0;
+    }
+
+    return TRUE;
+}
+
 void readline_init()
 {
     rl_attempted_completion_function = readline_on_complete;
@@ -2251,7 +1955,15 @@ void readline_init()
     gEditingDir = STRING_NEW("");
     gMethodCompletions = NULL;
 
-    rl_bind_key('x'-'a'+1, readline_select_file);
+    rl_bind_key('x'-'a'+1, readline_macro);
+    //rl_bind_key('x'-'a'+1, readline_select_file);
+    
+    saphire_add_inner_command("history", commands_history);
+    saphire_add_inner_command("add_history", commands_add_history);
+    saphire_add_inner_command("read_history", commands_read_history);
+    saphire_add_inner_command("write_history", commands_write_history);
+    saphire_add_inner_command("clear_history", commands_clear_history);
+    saphire_add_inner_command("remove_history", commands_remove_history);
 }
 
 void readline_final()
