@@ -37,7 +37,7 @@ hash_obj* hash_new(int size);
 #define HASH_NEW(size) hash_new(size)
 
 
-void hash_delete(hash_obj* self);               // 
+//void hash_delete(hash_obj* self);               // 
 
 void hash_put(hash_obj* self, char* key, void* item);       // 
 void hash_erase(hash_obj* self, char* key);                 // 폜
