@@ -60,6 +60,8 @@ enum eCommandKind {
     kEnvSpace,
     kQuote,
     kTime,
+    kFSelector,
+    kCommandHistory,
     kInnerCommand,
     kCommand 
 };
@@ -385,6 +387,16 @@ string_obj* sGlobal_new(char* str);
 vector_obj* sAry_new(int first_size);
 hash_obj* sHash_new(int size);
 
+typedef struct {
+    sStatments* mStatments;
+    enum eLineField mLineField;
+} sAtCommand;
+
+sAtCommand* sAtCommand_new(sStatments* statments, enum eLineField lf);
+sAtCommand* sAtCommand_new2(sAtCommand* at_command);
+sAtCommand* sAtCommand_save(sAtCommand* self, int fd);
+sAtCommand* sAtCommand_load(int fd);
+
 ////////////////////////////////////////////////////////////
 /// from saphire_commands.c
 ////////////////////////////////////////////////////////////
