@@ -94,6 +94,8 @@ char* gStatmentKindStrs[kCommand+1] = {
     "quote",
     "time",
 //    "parse",
+    "fselector",
+    "cmdhistory",
     "inner_command",
     "command"
 };
@@ -938,7 +940,7 @@ sArg* sArg_new2(sArg* arg)
         break;
 
     case 4:
-        self->mBody = sStatments_new2(arg->mBody);
+        self->mBody = sAtCommand_new2(arg->mBody);
         break;
 
     case 5:
@@ -994,7 +996,7 @@ void sArg_save(sArg* self, int fd)
             break;
 
         case 4:
-            sStatments_save(self->mBody, fd);
+            sAtCommand_save(self->mBody, fd);
             break;
 
         case 5:
@@ -1065,7 +1067,7 @@ sArg* sArg_load(int fd)
         break;
 
     case 4:
-        self->mBody = sStatments_load(fd);
+        self->mBody = sAtCommand_load(fd);
         break;
 
     case 5:
@@ -2540,7 +2542,7 @@ void saphire_init(enum eAppType app_type, BOOL job_control, enum eRuntimeScript
 
     gReadBuffers = HASH_NEW(100);
 
-    setenv("SAPHIRE_VERSION", "3.4.8", 1);
+    setenv("SAPHIRE_VERSION", "3.5.0", 1);
     setenv("SAPHIRE_PROMPT", "> ", 1);
     setenv("SAPHIRE_DOCDIR", DOCDIR, 1);
     char* home = getenv("HOME");
@@ -4037,3 +4039,51 @@ sRef* sRef_new(void* mem, enum eMemCheckerKind kind)
 
     return self;
 }
+
+sAtCommand* sAtCommand_new(sStatments* statments, enum eLineField lf)
+{
+    sAtCommand* self = MALLOC(sizeof(sAtCommand));
+
+    self->mStatments = statments;
+    self->mLineField = lf;
+    
+    return self;
+}
+
+sAtCommand* sAtCommand_new2(sAtCommand* at_command)
+{
+    sAtCommand* self = MALLOC(sizeof(sAtCommand));
+    
+    self->mStatments = sStatments_new2(at_command->mStatments);
+    self->mLineField = at_command->mLineField;
+
+    return self;
+}
+
+sAtCommand* sAtCommand_save(sAtCommand* self, int fd)
+{
+    sStatments_save(self->mStatments, fd);
+
+    int n = self->mLineField;
+    if(write(fd, &n, sizeof(int)) < 0) {
+        perror("write");
+        exit(1);
+    }
+}
+
+sAtCommand* sAtCommand_load(int fd)
+{
+    sAtCommand* self = MALLOC(sizeof(sAtCommand));
+
+    self->mStatments = sStatments_load(fd);
+
+    int n;
+    if(read(fd, &n, sizeof(int)) < 0) {
+        perror("read");
+        exit(1);
+    }
+    self->mLineField = n;
+
+    return self;
+}
+
